/**
 * WordPress Dependencies
 */
import { select, subscribe } from "@wordpress/data";
import { isFunction } from "lodash";
import { parse, isReusableBlock, isTemplatePart } from "@wordpress/blocks";

// internal dependencies
import { mimmikCssForResponsivePreview } from "./helpers";

//Generate Styles
(() => {
  const { editor_type } = eb_style_handler;

  const makeIdOfTemplatePart = (theme, slug) =>
    theme && slug ? theme + "//" + slug : null;
  const emptyFuns = () => {};

  let ebEditGetPreviewDeviceType = () => {};
  let storeObjToGetPreviewType = {};

  if (editor_type === "edit-site") {
    storeObjToGetPreviewType = select("core/edit-site");
  } else if (editor_type === "edit-post") {
    storeObjToGetPreviewType = select("core/edit-post");
  }

  ebEditGetPreviewDeviceType =
    storeObjToGetPreviewType.__experimentalGetPreviewDeviceType;

  // // error check
  // console.log({ebEditGetPreviewDeviceType})

  window.ebEditCurrentPreviewOption = ebEditGetPreviewDeviceType();

  const {
    isSavingPost = emptyFuns,
    isAutosavingPost = emptyFuns,
    isSavingNonPostEntityChanges = emptyFuns,
  } = select("core/editor");

  const regexForBlockNameTest = /^essential\-blocks\//g;

  //
  const generateAndSendStyleToBackend = () => {
    const { getEditedEntityRecord = emptyFuns } = select("core");
    const { getBlocks = emptyFuns } = select("core/block-editor");
    const { getCurrentPostId = emptyFuns } = select("core/editor");
    const post_id = getCurrentPostId();
    const all_blocks = getBlocks();

    const styles = {};
    const cssObjectMaker = (blocks) => {
      for (const item of blocks) {
        const {
          attributes: { blockMeta, blockRoot, blockId },
          innerBlocks,
        } = item;
        if (isFunction(isReusableBlock) && isReusableBlock(item)) {
          const reusableBlock =
            getEditedEntityRecord(
              "postType",
              "wp_block",
              item.attributes.ref
            ) || {};

          const parsedBlocks = reusableBlock.content
            ? parse(
                isFunction(reusableBlock.content)
                  ? reusableBlock.content(reusableBlock)
                  : reusableBlock.content
              )
            : [];

          for (const blockItem of parsedBlocks) {
            const {
              attributes: { blockMeta, blockRoot, blockId },
              innerBlocks,
            } = blockItem;
            if (blockMeta && blockRoot === "essential_block") {
              styles[blockId] = blockMeta;
            }
            if (innerBlocks.length > 0) {
              cssObjectMaker(innerBlocks);
            }
          }
        } else if (isFunction(isTemplatePart) && isTemplatePart(item)) {
          const { theme, slug } = item.attributes;

          const templatePartEntity =
            getEditedEntityRecord(
              "postType",
              "wp_template_part",
              makeIdOfTemplatePart(theme, slug)
            ) || {};

          const { blocks = [], innerBlocks = [] } = templatePartEntity;
          cssObjectMaker(blocks);
          cssObjectMaker(innerBlocks);
        } else if (blockMeta && blockRoot === "essential_block") {
          styles[blockId] = blockMeta;
        }
        if (innerBlocks.length > 0) {
          cssObjectMaker(innerBlocks);
        }
      }
    };

    cssObjectMaker(all_blocks); // Call function

    const stringStyles = JSON.stringify(styles);

    jQuery.ajax({
      type: "POST",
      url: ajaxurl,
      data: {
        data: stringStyles,
        id: post_id,
        editorType: editor_type,
        action: "eb_write_block_css",
        nonce: eb_style_handler.sth_nonce,
      },
      error: function (msg) {
        console.log(msg);
      },
    });
  };

  const callBackFuncOnPreviewChange = () => {
    // console.log(
    //   "--ebEditCurrentPreviewOption: ",
    //   ebEditCurrentPreviewOption,
    //   "--ebEditGetPreviewDeviceType: ",
    //   ebEditGetPreviewDeviceType()
    // );

    if (window.ebEditCurrentPreviewOption !== ebEditGetPreviewDeviceType()) {
      // console.log("----1. passed if check to update blocks resOtion ");
      const newDeviceType = ebEditGetPreviewDeviceType();

      // the following line of code should be at the top of this if statement
      window.ebEditCurrentPreviewOption = newDeviceType;

      //
      mimmikCssForResponsivePreview({
        domObj: document,
        resOption: newDeviceType,
      });

      //
      const { isFunction } = lodash;
      const {
        parse = emptyFuns,
        isReusableBlock = emptyFuns,
        isTemplatePart = emptyFuns,
      } = wp.blocks;
      const { getEditedEntityRecord = emptyFuns } = select("core");
      const {
        getBlocks = emptyFuns,
        updateBlockAttributes = emptyFuns,
      } = select("core/block-editor");

      const all_blocks = getBlocks();

      const resOptionChanger = (blocks) => {
        // console.log(
        //   "----2. entered inside resOptionChanger Function with all blocks:",
        //   { blocks }
        // );

        for (const item of blocks) {
          const { name = "", clientId, innerBlocks = [] } = item;

          if (isFunction(isReusableBlock) && isReusableBlock(item)) {
            const reusableBlock =
              getEditedEntityRecord(
                "postType",
                "wp_block",
                item.attributes.ref
              ) || {};

            const parsedBlocks = reusableBlock.content
              ? parse(
                  isFunction(reusableBlock.content)
                    ? reusableBlock.content(reusableBlock)
                    : reusableBlock.content
                )
              : [];

            resOptionChanger(parsedBlocks);
          } else if (isFunction(isTemplatePart) && isTemplatePart(item)) {
            const { theme, slug } = item.attributes;

            const templatePartEntity =
              getEditedEntityRecord(
                "postType",
                "wp_template_part",
                makeIdOfTemplatePart(theme, slug)
              ) || {};

            const { blocks = [], innerBlocks = [] } = templatePartEntity;
            resOptionChanger(blocks);
            resOptionChanger(innerBlocks);
          } else if (regexForBlockNameTest.test(name)) {
            updateBlockAttributes(clientId, {
              resOption: newDeviceType,
            });
            // console.log("----3. done  updateBlockAttributes (resOption):", {
            //   clientId,
            //   updateBlockAttributes,
            //   newDeviceType,
            // });
          }

          if (innerBlocks.length > 0) {
            resOptionChanger(innerBlocks);
          }
        }
      };

      resOptionChanger(all_blocks); // Call function

      // console.log("----last: passed if check to update blocks resOtion ");
    }
  };

  //
  const callBackFuncForSubsCribeForEditPost = () => {
    if (isSavingPost() && !isAutosavingPost()) {
      generateAndSendStyleToBackend();
    }
    callBackFuncOnPreviewChange();
  };

  const callBackFuncForSubsCribeForEditSite = () => {
    if (isSavingNonPostEntityChanges()) {
      generateAndSendStyleToBackend();
    }
    callBackFuncOnPreviewChange();
  };

  if (editor_type === "edit-site") {
    //
    subscribe(callBackFuncForSubsCribeForEditSite);
  } else if (editor_type === "edit-post") {
    //
    subscribe(callBackFuncForSubsCribeForEditPost);
  }
})();
